@@ -32,6 +32,8 @@ import flask_login
 import flask_principal
 from flask_babel import gettext, lazy_gettext
 
+from sqlalchemy import or_
+
 import ipranges
 import pynspect.gparser
 import pynspect.traversers
@@ -49,7 +51,7 @@ from vial.app import VialBlueprint
 from vial.view import RenderableView, ItemListView, ItemShowView, ItemCreateView, ItemCreateForView, ItemUpdateView, ItemDeleteView, ItemEnableView, ItemDisableView
 from vial.view.mixin import HTMLMixin, SQLAlchemyMixin
 import hawat.events
-from hawat.blueprints.filters.forms import BaseFilterForm, AdminFilterForm, PlaygroundFilterForm
+from hawat.blueprints.filters.forms import BaseFilterForm, AdminFilterForm, PlaygroundFilterForm, FilterSearchForm
 
 
 _PARSER = PynspectFilterParser()
@@ -196,6 +198,62 @@ class ListView(HTMLMixin, SQLAlchemyMixin, ItemListView):
         )
         return action_menu
 
+    @staticmethod
+    def get_search_form(request_args):
+        """
+        Must return instance of :py:mod:`flask_wtf.FlaskForm` appropriate for
+        searching given type of items.
+        """
+        return FilterSearchForm(
+            request_args,
+            meta = {'csrf': False}
+        )
+
+    @staticmethod
+    def build_query(query, model, form_args):
+        # Adjust query based on text search string.
+        if 'search' in form_args and form_args['search']:
+            query = query\
+                .filter(
+                    or_(
+                        model.name.like('%{}%'.format(form_args['search'])),
+                        model.filter.like('%{}%'.format(form_args['search'])),
+                        model.description.like('%{}%'.format(form_args['search'])),
+                    )
+                )
+        # Adjust query based on lower time boudary selection.
+        if 'dt_from' in form_args and form_args['dt_from']:
+            query = query.filter(model.createtime >= form_args['dt_from'])
+        # Adjust query based on upper time boudary selection.
+        if 'dt_to' in form_args and form_args['dt_to']:
+            query = query.filter(model.createtime <= form_args['dt_to'])
+        # Adjust query based on item state selection.
+        if 'state' in form_args and form_args['state']:
+            if form_args['state'] == 'enabled':
+                query = query.filter(model.enabled == True)
+            elif form_args['state'] == 'disabled':
+                query = query.filter(model.enabled == False)
+        # Adjust query based on upper time boudary selection.
+        if 'type' in form_args and form_args['type']:
+            query = query.filter(model.type == form_args['type'])
+        # Adjust query based on user membership selection.
+        if 'group' in form_args and form_args['group']:
+            query = query\
+                .filter(model.group_id == form_args['group'].id)
+        if 'sortby' in form_args and form_args['sortby']:
+            sortmap = {
+                'createtime.desc': lambda x, y: x.order_by(y.createtime.desc()),
+                'createtime.asc': lambda x, y: x.order_by(y.createtime.asc()),
+                'name.desc': lambda x, y: x.order_by(y.name.desc()),
+                'name.asc': lambda x, y: x.order_by(y.name.asc()),
+                'hits.desc': lambda x, y: x.order_by(y.hits.desc()),
+                'hits.asc': lambda x, y: x.order_by(y.hits.asc()),
+                'last_hit.desc': lambda x, y: x.order_by(y.last_hit.desc()),
+                'last_hit.asc': lambda x, y: x.order_by(y.last_hit.asc())
+            }
+            query = sortmap[form_args['sortby']](query, model)
+        return query
+
 
 class ShowView(HTMLMixin, SQLAlchemyMixin, ItemShowView):
     """
