@@ -209,3 +209,92 @@ class PlaygroundFilterForm(flask_wtf.FlaskForm):
     submit = wtforms.SubmitField(
         lazy_gettext('Check')
     )
+
+class FilterSearchForm(vial.forms.BaseSearchForm):
+    """
+    Class representing simple user search form.
+    """
+    search = wtforms.StringField(
+        lazy_gettext('Netname, network, description:'),
+        validators = [
+            wtforms.validators.Optional(),
+            wtforms.validators.Length(min = 3, max = 100)
+        ],
+        description = lazy_gettext('Filter`s name, content or description. Search is performed even in the middle of the strings.')
+    )
+    dt_from = vial.forms.SmartDateTimeField(
+        lazy_gettext('Creation time from:'),
+        validators = [
+            wtforms.validators.Optional()
+        ],
+        description = lazy_gettext('Lower time boundary for item creation time. Timestamp is expected to be in the format <code>YYYY-MM-DD hh:mm:ss</code> and in the timezone according to the user`s preferences.')
+    )
+    dt_to = vial.forms.SmartDateTimeField(
+        lazy_gettext('Creation time to:'),
+        validators = [
+            wtforms.validators.Optional()
+        ],
+        description = lazy_gettext('Upper time boundary for item creation time. Timestamp is expected to be in the format <code>YYYY-MM-DD hh:mm:ss</code> and in the timezone according to the user`s preferences.')
+    )
+
+    type = wtforms.SelectField(
+        lazy_gettext('Type:'),
+        validators = [
+            wtforms.validators.Optional(),
+        ],
+        choices = [
+            ('', lazy_gettext('Nothing selected')),
+            (REPORTING_FILTER_BASIC,    lazy_gettext('Basic')),
+            (REPORTING_FILTER_ADVANCED, lazy_gettext('Advanced'))
+        ],
+        default = '',
+        description = lazy_gettext('Search for filters of particular type.')
+    )
+    state = wtforms.SelectField(
+        lazy_gettext('State:'),
+        validators = [
+            wtforms.validators.Optional(),
+        ],
+        choices = [
+            ('', lazy_gettext('Nothing selected')),
+            ('enabled',  lazy_gettext('Enabled')),
+            ('disabled', lazy_gettext('Disabled'))
+        ],
+        default = '',
+        description = lazy_gettext('Search for filters with particular state.')
+    )
+    group = QuerySelectField(
+        lazy_gettext('Group:'),
+        query_factory = get_available_groups,
+        allow_blank = True,
+        description = lazy_gettext('Search for filters belonging to particular group.')
+    )
+
+    sortby = wtforms.SelectField(
+        lazy_gettext('Sort by:'),
+        validators = [
+            wtforms.validators.Optional()
+        ],
+        choices = [
+            ('createtime.desc', lazy_gettext('by creation time descending')),
+            ('createtime.asc',  lazy_gettext('by creation time ascending')),
+            ('name.desc', lazy_gettext('by netname descending')),
+            ('name.asc',  lazy_gettext('by netname ascending')),
+            ('hits.desc', lazy_gettext('by number of hits descending')),
+            ('hits.asc',  lazy_gettext('by number of hits ascending')),
+            ('last_hit.desc', lazy_gettext('by time of last hit descending')),
+            ('last_hit.asc',  lazy_gettext('by time of last hit ascending'))
+        ],
+        default = 'name.asc'
+    )
+
+    @staticmethod
+    def is_multivalue(field_name):
+        """
+        Check, if given form field is a multivalue field.
+
+        :param str field_name: Name of the form field.
+        :return: ``True``, if the field can contain multiple values, ``False`` otherwise.
+        :rtype: bool
+        """
+        return False
