@@ -29,13 +29,15 @@ import flask_login
 import flask_principal
 from flask_babel import gettext, lazy_gettext
 
+from sqlalchemy import or_
+
 from mentat.datatype.sqldb import NetworkModel, GroupModel, ItemChangeLogModel
 
 import vial.acl
 from vial.app import VialBlueprint
 from vial.view import ItemListView, ItemShowView, ItemCreateView, ItemCreateForView, ItemUpdateView, ItemDeleteView
 from vial.view.mixin import HTMLMixin, SQLAlchemyMixin
-from hawat.blueprints.networks.forms import BaseNetworkForm, AdminNetworkForm
+from hawat.blueprints.networks.forms import BaseNetworkForm, AdminNetworkForm, NetworkSearchForm
 
 
 BLUEPRINT_NAME = 'networks'
@@ -94,6 +96,55 @@ class ListView(HTMLMixin, SQLAlchemyMixin, ItemListView):
         )
         return action_menu
 
+    @staticmethod
+    def get_search_form(request_args):
+        """
+        Must return instance of :py:mod:`flask_wtf.FlaskForm` appropriate for
+        searching given type of items.
+        """
+        return NetworkSearchForm(
+            request_args,
+            meta = {'csrf': False}
+        )
+
+    @staticmethod
+    def build_query(query, model, form_args):
+        # Adjust query based on text search string.
+        if 'search' in form_args and form_args['search']:
+            query = query\
+                .filter(
+                    or_(
+                        model.netname.like('%{}%'.format(form_args['search'])),
+                        model.network.like('%{}%'.format(form_args['search'])),
+                        model.description.like('%{}%'.format(form_args['search'])),
+                    )
+                )
+        # Adjust query based on lower time boudary selection.
+        if 'dt_from' in form_args and form_args['dt_from']:
+            query = query.filter(model.createtime >= form_args['dt_from'])
+        # Adjust query based on upper time boudary selection.
+        if 'dt_to' in form_args and form_args['dt_to']:
+            query = query.filter(model.createtime <= form_args['dt_to'])
+        # Adjust query based on record source selection.
+        if 'source' in form_args and form_args['source']:
+            query = query\
+                .filter(model.source == form_args['source'])
+        # Adjust query based on user membership selection.
+        if 'group' in form_args and form_args['group']:
+            query = query\
+                .filter(model.group_id == form_args['group'].id)
+        if 'sortby' in form_args and form_args['sortby']:
+            sortmap = {
+                'createtime.desc': lambda x, y: x.order_by(y.createtime.desc()),
+                'createtime.asc': lambda x, y: x.order_by(y.createtime.asc()),
+                'netname.desc': lambda x, y: x.order_by(y.netname.desc()),
+                'netname.asc': lambda x, y: x.order_by(y.netname.asc()),
+                'network.desc': lambda x, y: x.order_by(y.network.desc()),
+                'network.asc': lambda x, y: x.order_by(y.network.asc())
+            }
+            query = sortmap[form_args['sortby']](query, model)
+        return query
+
 
 class ShowView(HTMLMixin, SQLAlchemyMixin, ItemShowView):
     """
