@@ -105,8 +105,6 @@ class SearchView(HTMLMixin, SQLAlchemyMixin, BaseSearchView):  # pylint: disable
 
     authentication = True
 
-    authorization = [vial.acl.PERMISSION_ANY]
-
     @classmethod
     def get_view_icon(cls):
         return 'module-{}'.format(cls.module_name)
@@ -141,7 +139,7 @@ class SearchView(HTMLMixin, SQLAlchemyMixin, BaseSearchView):  # pylint: disable
         if 'dt_to' in form_args and form_args['dt_to']:
             query = query.filter(model.createtime <= form_args['dt_to'])
         # Adjust query based on report severity selection.
-        if 'severities' in form_args and form_args['severities']:
+        if 'types' in form_args and form_args['severities']:
             query = query.filter(model.severity.in_(form_args['severities']))
         # Adjust query based on report type selection.
         if 'severities' in form_args and form_args['types']:
@@ -387,8 +385,6 @@ class DashboardView(HTMLMixin, SQLAlchemyMixin, BaseSearchView):  # pylint: disa
     """
     authentication = True
 
-    authorization = [vial.acl.PERMISSION_ANY]
-
     @classmethod
     def get_view_name(cls):
         return 'dashboard'
@@ -523,8 +519,6 @@ class FeedbackView(AJAXMixin, RenderableView):
 
     authentication = True
 
-    authorization = [vial.acl.PERMISSION_ANY]
-
     @classmethod
     def get_view_name(cls):
         return 'feedback'
