@@ -66,10 +66,13 @@ from sqlalchemy.ext.declarative import declarative_base, declared_attr
 from sqlalchemy.schema import DropTable
 from sqlalchemy.ext.compiler import compiles
 
+from werkzeug.security import generate_password_hash, check_password_hash
+
 from mentat.const import REPORTING_ATTACHS, REPORTING_MODES, REPORTING_TIMINGS,\
     REPORTING_FILTERS, REPORTING_FILTER_BASIC,\
     REPORT_TYPES, REPORT_SEVERITIES
 
+
 #
 # Modify compilation of DROP TABLE for PostgreSQL databases to enable CASCADE feature.
 # Otherwise it is not possible to delete the database schema with:
@@ -178,6 +181,7 @@ class UserModel(MODEL):
     email        = sqlalchemy.Column(sqlalchemy.String(250), nullable = False)
     organization = sqlalchemy.Column(sqlalchemy.String(250), nullable = False)
     roles        = sqlalchemy.Column(sqlalchemy.dialects.postgresql.ARRAY(sqlalchemy.String(20), dimensions = 1), nullable = False, default = [])
+    password     = sqlalchemy.Column(sqlalchemy.String)
     apikey       = sqlalchemy.Column(sqlalchemy.String, unique = True, index = True)
     enabled      = sqlalchemy.Column(sqlalchemy.Boolean, nullable = False, default = True)
 
@@ -222,23 +226,36 @@ class UserModel(MODEL):
         """
         self.enabled = False
 
+    def set_password(self, password_plain):
+        """
+        Generate and set password hash from given plain text password.
+        """
+        self.password = generate_password_hash(password_plain)
+
+    def check_password(self, password_plain):
+        """
+        Check given plaintext password agains internal password hash.
+        """
+        return check_password_hash(self.password, password_plain)
+
     def to_dict(self):
         """
         *Interface implementation:* Implementation of :py:func:`mentat.datatype.sqldb.MODEL.to_dict` method.
         """
         return {
-            'id':                 int(self.id),
+            'id':                 self.id,
             'createtime':         str(self.createtime),
             'logintime':          str(self.logintime),
-            'login':              str(self.login),
-            'fullname':           str(self.fullname),
-            'email':              str(self.email),
-            'organization':       str(self.organization),
+            'login':              self.login,
+            'fullname':           self.fullname,
+            'email':              self.email,
+            'organization':       self.organization,
             'roles':              [ str(x) for x in self.roles],
-            'apikey':             str(self.apikey),
+            'apikey':             self.apikey,
+            'password':           self.password,
             'enabled':            bool(self.enabled),
-            'locale':             str(self.locale),
-            'timezone':           str(self.timezone),
+            'locale':             self.locale,
+            'timezone':           self.timezone,
             'memberships':        [(x.id, x.name) for x in self.memberships],
             'memberships_wanted': [(x.id, x.name) for x in self.memberships_wanted],
             'managements':        [(x.id, x.name) for x in self.managements]
@@ -293,6 +310,7 @@ class UserModel(MODEL):
         """
         return len(self.roles) == 0
 
+
 def usermodel_from_typeddict(structure, defaults = None):
     """
     Convenience method for creating :py:class:`mentat.datatype.sqldb.UserModel`
@@ -479,6 +497,7 @@ class FilterModel(MODEL):  # pylint: disable=locally-disabled,too-many-instance-
     categories  = sqlalchemy.Column(sqlalchemy.dialects.postgresql.ARRAY(sqlalchemy.String, dimensions = 1), default = [], nullable = False)
     sources     = sqlalchemy.Column(sqlalchemy.dialects.postgresql.ARRAY(sqlalchemy.String, dimensions = 1), default = [], nullable = False)
     hits        = sqlalchemy.Column(sqlalchemy.Integer, default = 0, nullable = False)
+    last_hit    = sqlalchemy.Column(sqlalchemy.DateTime)
 
     def __repr__(self):
         return "<Filter(name='%s')>" % (self.name)
@@ -528,7 +547,8 @@ class FilterModel(MODEL):  # pylint: disable=locally-disabled,too-many-instance-
             'detectors':   [str(x) for x in self.detectors],
             'categories':  [str(x) for x in self.categories],
             'sources':     [str(x) for x in self.sources],
-            'hits':        int(self.hits)
+            'hits':        int(self.hits),
+            'last_hit':    str(self.last_hit)
         }
 
 def filtermodel_from_typeddict(structure, defaults = None):
@@ -553,6 +573,7 @@ def filtermodel_from_typeddict(structure, defaults = None):
     sqlobj.categories  = structure.get('categories', [])
     sqlobj.sources     = structure.get('ips', [])
     sqlobj.hits        = structure.get('hits', 0)
+    sqlobj.last_hit    = structure.get('lasthit', None)
 
     return sqlobj
 
@@ -595,7 +616,7 @@ class SettingsReportingModel(MODEL):  # pylint: disable=locally-disabled,too-few
     max_attachment_size = sqlalchemy.Column(sqlalchemy.Integer)
 
     def __repr__(self):
-        return "<SettingsReporting(id='%d',group_id='%d')>" % (self.id, self.group_id)
+        return "<SettingsReporting(id='%d',group_id='%d')>" % (int(self.id), int(self.group_id))
 
     def to_dict(self):
         """
@@ -896,7 +917,7 @@ def jsondiff(json_obj_a, json_obj_b):
     Calculate the difference between two model objects given as JSON strings.
     """
     return "\n".join(
-        [l for l in difflib.unified_diff(json_obj_a.split("\n"), json_obj_b.split("\n"))]
+        difflib.unified_diff(json_obj_a.split("\n"), json_obj_b.split("\n"))
     )
 
 def dictdiff(dict_obj_a, dict_obj_b):
