@@ -306,22 +306,18 @@ class ThresholdingCache:
     reporting.
     """
 
-    def event_is_thresholded(self, event, ttl):
+    def event_is_thresholded(self, event, source, ttl):
         """
-        Check, that given event is thresholded within given TTL.
+        Check, that given combination of event and source is thresholded within given TTL.
 
         :param mentat.idea.internal.Idea event: IDEA event to check.
+        :param str source: Source to check.
         :param datetime.datetime ttl: TTL for the thresholding record.
         :return: ``True`` in case the event is thresholded, ``False`` otherwise.
         :rtype: bool
         """
-        cachekeys = self._generate_cache_keys(event)
-        result = True
-        for chk in cachekeys:
-            res = self.check(chk, ttl)
-            if not res:
-                result = False
-        return result
+        cachekey = self._generate_cache_key(event, source)
+        return self.check(cachekey, ttl)
 
     def set_threshold(self, event, source, thresholdtime, relapsetime, ttl):
         """
@@ -333,24 +329,21 @@ class ThresholdingCache:
         :param datetime.datetime relapsetime: Relapse window start time.
         :param datetime.datetime ttl: Record TTL.
         """
-        if source:
-            source = [source]
-        cachekeys = self._generate_cache_keys(event, source)
-        for chk in cachekeys:
-            self.set(chk, thresholdtime, relapsetime, ttl)
+        cachekey = self._generate_cache_key(event, source)
+        self.set(cachekey, thresholdtime, relapsetime, ttl)
 
-    def threshold_event(self, event, group_name, severity, createtime):
+    def threshold_event(self, event, source, group_name, severity, createtime):
         """
         Threshold given event with given TTL.
 
         :param mentat.idea.internal.Idea event: IDEA event to threshold.
+        :param str source: Source address because of which to threshold the event.
         :param str group_name: Name of the group for which to threshold.
         :param str severity: Event severity.
         :param datetime.datetime createtime: Thresholding timestamp.
         """
-        cachekeys = self._generate_cache_keys(event)
-        for chk in cachekeys:
-            self.save(event.get_id(), chk, group_name, severity, createtime)
+        cachekey = self._generate_cache_key(event, source)
+        self.save(event.get_id(), cachekey, group_name, severity, createtime)
 
     #---------------------------------------------------------------------------
 
@@ -394,8 +387,8 @@ class ThresholdingCache:
         :param str group_name: Name of the abuse group.
         :param str severity: Event severity.
         :param datetime.datetime ttl: Record TTL time.
-        :return: List of relapsed events as :py:class:`mentat.idea.internal.Idea` objects.
-        :rtype: list
+        :return: Touple with list of relapsed events as :py:class:`mentat.idea.internal.Idea` objects and their aggregation by keyid.
+        :rtype: touple
         """
         raise NotImplementedError()
 
@@ -409,25 +402,29 @@ class ThresholdingCache:
 
     #---------------------------------------------------------------------------
 
-    def _generate_cache_keys(self, event, sources = None):
+    def _generate_cache_key(self, event, source):
         """
-        Generate list of thresholding/relapse cache keys for given event.
+        Generate cache key for given event and source.
 
         :param mentat.idea.internal.Idea event: Event to process.
-        :return: List of cache keys as strings.
-        :rtype: list
+        :param str source: Source to process.
+        :return: Cache key as strings.
+        :rtype: str
         """
-        keys = []
         event_class = jpath_value(event, '_CESNET.EventClass')
         if not event_class:
             event_class = '/'.join(jpath_values(event, 'Category'))
-        if not sources:
-            sources_ip4 = jpath_values(event, 'Source.IP4')
-            sources_ip6 = jpath_values(event, 'Source.IP6')
-            sources = sources_ip4 + sources_ip6
-        for src in sources:
-            keys.append('+++'.join((event_class, str(src))))
-        return keys
+        return '+++'.join((event_class, str(source)))
+
+    def get_source_from_cache_key(self, key):
+        """
+        Return source from which was key generated.
+
+        :param str key: Cache key.
+        :return: Cached source.
+        :rtype: str
+        """
+        return key.split('+++')[1] if key and len(key.split('+++')) > 1 else key
 
 
 class NoThresholdingCache(ThresholdingCache):
@@ -504,7 +501,12 @@ class SimpleMemoryThresholdingCache(ThresholdingCache):
         """
         *Interface implementation:* Implementation of :py:func:`mentat.reports.utils.ThresholdingCache.cleanup` method.
         """
-        return {'thresholds': 0, 'events': 0}
+        result = {
+            'thresholds': len(self.memcache),
+            'events': 0
+        }
+        self.memcache = {}
+        return result
 
 
 class SingleSourceThresholdingCache(SimpleMemoryThresholdingCache):
@@ -517,15 +519,16 @@ class SingleSourceThresholdingCache(SimpleMemoryThresholdingCache):
         super().__init__()
         self.source = source
 
-    def _generate_cache_keys(self, event, sources = None):
+    def _generate_cache_key(self, event, source):
         """
-        Generate list of thresholding/relapse cache keys for given event.
+        Generate cache key for given event and source.
 
         :param mentat.idea.internal.Idea event: Event to process.
-        :return: List of cache keys as strings.
-        :rtype: list
+        :param str source: Source to process.
+        :return: Cache key as strings.
+        :rtype: str
         """
-        return super()._generate_cache_keys(event, [self.source])
+        return super()._generate_cache_key(event, self.source)
 
 
 class StorageThresholdingCache(ThresholdingCache):
