@@ -46,20 +46,20 @@ import psycopg2
 import psycopg2.extras
 import psycopg2.sql
 
-#
-# Custom libraries
-#
 import ipranges
 import mentat.idea.sqldb
 import mentat.idea.internal
 from mentat.const import CKEY_CORE_DATABASE, CKEY_CORE_DATABASE_EVENTSTORAGE, random_str
 
+
 _MANAGER = None
 
 QTYPE_SELECT       = 'select'
 QTYPE_SELECT_GHOST = 'select_ghost'
 QTYPE_COUNT        = 'count'
 QTYPE_DELETE       = 'delete'
+QTYPE_AGGREGATE    = 'aggregate'
+QTYPE_TIMELINE     = 'timeline'
 
 ENUM_TABLES = (
     "category",
@@ -99,9 +99,33 @@ EVENTS_COLUMNS = (
     "cesnet_inspectionerrors",
 )
 
+EVENTS_COLUMNS_ARRAY = (
+    "category",
+    "source_ip",
+    "target_ip",
+    "source_port",
+    "target_port",
+    "source_type",
+    "target_type",
+    "protocol",
+    "node_name",
+    "node_type",
+    "cesnet_resolvedabuses",
+    "cesnet_inspectionerrors",
+)
+
+EVENTS_COLUMNS_TOPLISTED = (
+    "source_ip",
+    "target_ip",
+    "source_port",
+    "target_port",
+    "cesnet_resolvedabuses",
+)
+
 RE_QNAME      = ' AS "_mentatq\\(([^)]+)\\)_"'
 RE_QNAME_CMPL = re.compile(RE_QNAME)
 
+
 class EventStorageException(Exception):
     """
     Class for custom event storage exceptions.
@@ -194,74 +218,79 @@ def _bq_searchby_addr(chunks, params, idents, items):
             items_exp.append(i)
     params.extend(items_exp)
 
-def _bq_qbase_select_full(qname = None):
+def _bq_qbase_select_full(parameters = None, qname = None):  # pylint: disable=locally-disabled,unused-argument
     query = psycopg2.sql.SQL('SELECT * FROM events')
     if qname:
         query += psycopg2.sql.SQL(' AS {}').format(
             psycopg2.sql.Identifier(qname)
         )
     query += psycopg2.sql.SQL(' INNER JOIN events_json USING(id)')
-    return query
+    return query, []
 
-def _bq_qbase_select_ghost(qname = None):
+def _bq_qbase_select_ghost(parameters = None, qname = None):  # pylint: disable=locally-disabled,unused-argument
     query = psycopg2.sql.SQL('SELECT * FROM events')
     if qname:
         query += psycopg2.sql.SQL(' AS {}').format(
             psycopg2.sql.Identifier(qname)
         )
-    return query
+    return query, []
 
-def _bq_qbase_count(qname = None):
+def _bq_qbase_count(parameters = None, qname = None):  # pylint: disable=locally-disabled,unused-argument
     query = psycopg2.sql.SQL('SELECT count(id) FROM events')
     if qname:
         query += psycopg2.sql.SQL(' AS {}').format(
             psycopg2.sql.Identifier(qname)
         )
-    return query
+    return query, []
 
-def _bq_qbase_delete(qname = None):
+def _bq_qbase_delete(parameters = None, qname = None):  # pylint: disable=locally-disabled,unused-argument
     query = psycopg2.sql.SQL('DELETE FROM events')
     if qname:
         query += psycopg2.sql.SQL(' AS {}').format(
             psycopg2.sql.Identifier(qname)
         )
-    return query
+    return query, []
+
+def _bq_qbase_aggregate(parameters = None, qname = None, query = None, params = None):  # pylint: disable=locally-disabled,unused-argument
+    if not query:
+        query = psycopg2.sql.SQL('SELECT')
+    if not params:
+        params = []
+    if parameters.get('aggr_set', None):
+        if parameters['aggr_set'] in EVENTS_COLUMNS_ARRAY:
+            query += psycopg2.sql.SQL(' unnest({}) AS set, COUNT(*) FROM events').format(
+                psycopg2.sql.Identifier(parameters['aggr_set'])
+            )
+        else:
+            query += psycopg2.sql.SQL(' {} AS set, COUNT(*) FROM events').format(
+                psycopg2.sql.Identifier(parameters['aggr_set'])
+            )
+    else:
+        query += psycopg2.sql.SQL(' COUNT(*) FROM events')
+    if qname:
+        query += psycopg2.sql.SQL(' AS {}').format(
+            psycopg2.sql.Identifier(qname)
+        )
+    return query, params
+
+def _bq_qbase_timeline(parameters = None, qname = None):  # pylint: disable=locally-disabled,unused-argument
+    params = []
+    query = psycopg2.sql.SQL('SELECT %s + %s * (width_bucket(detecttime, (SELECT array_agg(buckets) FROM generate_series(%s, %s, %s) AS buckets)) - 1) AS bucket,')
+    params.append(parameters['dt_from'])
+    params.append(parameters['step'])
+    params.append(parameters['dt_from'])
+    params.append(parameters['dt_to'])
+    params.append(parameters['step'])
+    return _bq_qbase_aggregate(parameters, qname, query, params)
 
 def _bq_qname_full(qname):
     return '_mentatq({})_'.format(qname)
 
-def build_query(parameters = None, qtype = QTYPE_SELECT, qname = None):  # pylint: disable=locally-disabled,too-many-branches
-    """
-    Build SQL database query according to given parameters.
-
-    :param dict parameters: Query parametersas complex dictionary structure.
-    :param str qtype: Type of the generated query ('select','count','delete').
-    :param str qname: Unique name for the generated query.
-    :return: Generated query as ``psycopg2.sql.SQL`` and apropriate arguments.
-    :rtype: tuple
-    """
+def _bq_where(parameters):
+    query = None
     chunks = []
     params = []
-    query  = None
 
-    if qname:
-        qname = _bq_qname_full(qname)
-
-    # Prepare query base.
-    if qtype == QTYPE_SELECT:
-        query = _bq_qbase_select_full(qname)
-    elif qtype == QTYPE_SELECT_GHOST:
-        query = _bq_qbase_select_ghost(qname)
-    elif qtype == QTYPE_COUNT:
-        query = _bq_qbase_count(qname)
-    elif qtype == QTYPE_DELETE:
-        query = _bq_qbase_delete(qname)
-    elif isinstance(qtype, psycopg2.sql.Composed):
-        query = qtype
-    else:
-        raise ValueError("Received invalid value '{}' for SQL query type.".format(qtype))
-
-    # Process and append query filtering parameters.
     if parameters:
         if parameters.get('dt_from', None):
             chunks.append(psycopg2.sql.SQL('{} >= %s').format(psycopg2.sql.Identifier('detecttime')))
@@ -283,6 +312,7 @@ def build_query(parameters = None, qtype = QTYPE_SELECT, qname = None):  # pylin
                 _bq_searchby_addr(chunks, params, ['source_ip'], parameters['source_addrs'])
             if parameters.get('target_addrs', None):
                 _bq_searchby_addr(chunks, params, ['target_ip'], parameters['target_addrs'])
+
         if parameters.get('host_ports', None):
             chunks.append(psycopg2.sql.SQL('({} && %s OR {} && %s)').format(psycopg2.sql.Identifier('source_port'), psycopg2.sql.Identifier('target_port')))
             params.extend([parameters['host_ports'], [int(x) for x in parameters['host_ports']]])
@@ -293,6 +323,7 @@ def build_query(parameters = None, qtype = QTYPE_SELECT, qname = None):  # pylin
             if parameters.get('target_ports', None):
                 chunks.append(psycopg2.sql.SQL('{} && %s').format(psycopg2.sql.Identifier('target_port')))
                 params.append([int(x) for x in parameters['target_ports']])
+
         if parameters.get('host_types', None):
             chunks.append(psycopg2.sql.SQL('({} && %s OR {} && %s)').format(psycopg2.sql.Identifier('source_type'), psycopg2.sql.Identifier('target_type')))
             params.extend([parameters['host_types'], parameters['host_types']])
@@ -328,10 +359,64 @@ def build_query(parameters = None, qtype = QTYPE_SELECT, qname = None):  # pylin
             params.append(parameters['description'])
 
     if chunks:
-        query += psycopg2.sql.SQL(' WHERE ')
+        query = psycopg2.sql.SQL(' WHERE ')
         query += psycopg2.sql.SQL(' AND ').join(chunks)
 
-    # Process and append query sorting and limiting parameters.
+    return query, params
+
+_BQ_MAP = {
+    QTYPE_SELECT:       _bq_qbase_select_full,
+    QTYPE_SELECT_GHOST: _bq_qbase_select_ghost,
+    QTYPE_COUNT:        _bq_qbase_count,
+    QTYPE_DELETE:       _bq_qbase_delete,
+    QTYPE_AGGREGATE:    _bq_qbase_aggregate,
+    QTYPE_TIMELINE:     _bq_qbase_timeline,
+}
+
+def build_query(parameters = None, qtype = QTYPE_SELECT, qname = None):
+    """
+    Build SQL database query according to given parameters.
+
+    :param dict parameters: Query parametersas complex dictionary structure.
+    :param str qtype: Type of the generated query ('select','count','delete').
+    :param str qname: Unique name for the generated query.
+    :return: Generated query as ``psycopg2.sql.SQL`` and apropriate arguments.
+    :rtype: tuple
+    """
+    query  = None
+    params = []
+
+    if qname:
+        qname = _bq_qname_full(qname)
+
+    # Prepare query base based on the requested type of the query.
+    try:
+        query, params_ext = _BQ_MAP[str(qtype)](parameters, qname)
+        params.extend(params_ext)
+
+    except KeyError:
+        if isinstance(qtype, psycopg2.sql.Composed):
+            query = qtype
+        else:
+            raise ValueError("Received invalid value '{}' for SQL query type.".format(qtype))
+
+    # Build WHERE section of the query.
+    subquery, subparams = _bq_where(parameters)
+    if subquery:
+        query += subquery
+        params.extend(subparams)
+
+    # Process and append query grouping parameters for aggregation and timeline queries.
+    if qtype == QTYPE_AGGREGATE and parameters.get('aggr_set', None):
+        query += psycopg2.sql.SQL(' GROUP BY set')
+
+    elif qtype == QTYPE_TIMELINE:
+        if parameters.get('aggr_set', None):
+            query += psycopg2.sql.SQL(' GROUP BY bucket, set')
+        else:
+            query += psycopg2.sql.SQL(' GROUP BY bucket')
+
+    # Process and append query sorting and limiting parameters for select queries.
     if qtype in (QTYPE_SELECT, QTYPE_SELECT_GHOST) and parameters:
         if parameters.get('sortby', None):
             field, direction = parameters['sortby'].split('.')
@@ -344,6 +429,7 @@ def build_query(parameters = None, qtype = QTYPE_SELECT, qname = None):  # pylin
                     field = 'cesnet_storagetime'
                 else:
                     field = 'detecttime'
+
             if direction in ('asc',):
                 query += psycopg2.sql.SQL(' ORDER BY {} ASC').format(psycopg2.sql.Identifier(field))
             else:
@@ -352,11 +438,52 @@ def build_query(parameters = None, qtype = QTYPE_SELECT, qname = None):  # pylin
         if parameters.get('limit', None):
             query += psycopg2.sql.SQL(' LIMIT %s')
             params.append(int(parameters['limit']))
-
             if 'page' in parameters and parameters['page'] and int(parameters['page']) > 1:
                 query += psycopg2.sql.SQL(' OFFSET %s')
                 params.append((int(parameters['page']) - 1) * int(parameters['limit']))
 
+    elif qtype == QTYPE_AGGREGATE:
+        if parameters.get('limit', None) and parameters.get('aggr_set', None):
+            if parameters['aggr_set'] in EVENTS_COLUMNS_TOPLISTED:
+                query += psycopg2.sql.SQL(' ORDER BY COUNT(*) DESC LIMIT %s')
+                params.append(int(parameters['limit']))
+
+    elif qtype == QTYPE_TIMELINE:
+        query += psycopg2.sql.SQL(' ORDER BY bucket ASC')
+
+    return query, params
+
+
+def build_query_toplist(parameters = None, qtype = QTYPE_AGGREGATE, qname = None):
+    """
+    Build aggregation or timeline SQL queries with toplisting within the database.
+    """
+    # For query types other than timeline fall back to build_query() method.
+    subquery, subparams = build_query(parameters, qtype, qname)
+    if qtype != QTYPE_TIMELINE:
+        return subquery, subparams
+
+    if not parameters or not parameters.get('limit', None) or not parameters.get('aggr_set', None) or parameters['aggr_set'] not in EVENTS_COLUMNS_TOPLISTED:
+        return subquery, subparams
+
+    # Otherwise produce following crazyness:
+    #SELECT dist.bucket AS bucket, dist.set AS set, dist.count AS count
+    #FROM
+    #    (/* here comes the original query */) AS dist
+    #INNER JOIN
+    #    (SELECT unnest(events.{column}) AS set, COUNT(*) FROM events WHERE detecttime > {since} AND detecttime < {until} GROUP BY set ORDER BY COUNT(*) LIMIT {topN}) AS toplist
+    #USING (set);
+
+    params = []
+    query = psycopg2.sql.SQL('SELECT dist.bucket AS bucket, dist.set AS set, dist.count AS count FROM (')
+    query += subquery
+    params.extend(subparams)
+    query += psycopg2.sql.SQL(') AS dist INNER JOIN (')
+    subquery, subparams = build_query(parameters, QTYPE_AGGREGATE, qname)
+    query += subquery
+    params.extend(subparams)
+    query += psycopg2.sql.SQL(') AS toplist USING (set)')
+
     return query, params
 
 
@@ -505,6 +632,24 @@ class EventStorageCursor:
 
     #---------------------------------------------------------------------------
 
+    def query_direct(self, raw_query, idents = None, params = None):
+        """
+        Perform direct database query.
+
+        :param str raw_query: Raw SQL query. Will be converted to :py:class:`psycopg2.sql.SQL`.
+        :param list idents: Optional list of SQL identifiers, will be converted to :py:class:`psycopg2.sql.Identifier` and formatted into ``raw_query`` above.
+        :param list params: Optional list of SQL parameters, will be formatted into ``raw_query`` above.
+        """
+        query = psycopg2.sql.SQL(raw_query)
+        if idents:
+            idents = [psycopg2.sql.Identifier(i) for i in idents]
+            query = query.format(*idents)
+
+        self.lastquery = self.cursor.mogrify(query, params)
+
+        self.cursor.execute(query, params)
+        return self.cursor.fetchall()
+
     def count_events(self, parameters = None, qname = None):
         """
         Count the number of IDEA events in database. There is an option to assign given
@@ -519,7 +664,7 @@ class EventStorageCursor:
         query, params  = build_query(parameters, qtype = 'count', qname = qname)
         self.lastquery = self.cursor.mogrify(query, params)
         self.cursor.execute(query, params)
-        
+
         record = self.cursor.fetchone()
         if record:
             return record[0]
@@ -549,9 +694,33 @@ class EventStorageCursor:
         events_raw  = self.cursor.fetchall()
         return event_count, [event_factory(event) for event in events_raw]
 
+    def search_events_aggr(self, parameters = None, qtype = QTYPE_AGGREGATE, qname = None, dbtoplist = False):
+        """
+        Search IDEA events in database according to given parameters and perform selected aggregations. The
+        parameters will be passed down to the :py:func:`mentat.services.eventstorage.build_query`
+        function to generate proper SQL query. There is an option to assign given
+        unique name to the select query, so that it can be identified within the
+        ``pg_stat_activity`` table.
+
+        :param dict parameters: Search query parameters, see :py:func:`mentat.services.eventstorage.build_query` for details.
+        :param string qtype: Type of the select query.
+        :param str qname: Optional unique name for the generated query.
+        :return: Number of IDEA events in the result and list of events.
+        :rtype: tuple
+        """
+        if not dbtoplist:
+            query, params = build_query(parameters, qtype = qtype, qname = qname)
+        else:
+            query, params = build_query_toplist(parameters, qtype = qtype, qname = qname)
+        self.lastquery = self.cursor.mogrify(query, params)
+        print(self.lastquery)
+
+        self.cursor.execute(query, params)
+        return self.cursor.fetchall()
+
     def delete_events(self, parameters = None, qname = None):
         """
-        Delete IDEA events in database according to given parameters. There is 
+        Delete IDEA events in database according to given parameters. There is
         an option to assign given unique name to the query, so that it can be
         identified within the ``pg_stat_activity`` table.
 
@@ -717,15 +886,14 @@ class EventStorageCursor:
         :param str group_name: Name of the abuse group.
         :param str severity: Event severity.
         :param datetime.datetime ttl: Record TTL time.
-        :return: List of relapsed events as :py:class:`mentat.idea.internal.Idea` objects.
+        :return: List of relapsed events as touple of id, json of event data and list of threshold keys.
         :rtype: list
         """
         self.cursor.execute(
-            "SELECT events.*, events_json.event FROM events INNER JOIN events_json ON events.id = events_json.id WHERE events.id IN (SELECT DISTINCT eventid FROM events_thresholded WHERE keyid IN (SELECT keyid FROM events_thresholded INNER JOIN thresholds ON (events_thresholded.keyid = thresholds.id) WHERE events_thresholded.groupname = %s AND events_thresholded.eventseverity = %s AND events_thresholded.createtime >= thresholds.relapsetime AND thresholds.ttltime <= %s))",
+            "SELECT events_json.id, events_json.event, ARRAY_AGG(events_thresholded.keyid) AS keyids FROM events_json INNER JOIN events_thresholded ON events_json.id = events_thresholded.eventid INNER JOIN thresholds ON events_thresholded.keyid = thresholds.id WHERE events_thresholded.groupname = %s AND events_thresholded.eventseverity = %s AND events_thresholded.createtime >= thresholds.relapsetime AND thresholds.ttltime <= %s GROUP BY events_json.id",
             (group_name, severity, ttl)
         )
-        events_raw  = self.cursor.fetchall()
-        return [record_to_idea(event) for event in events_raw]
+        return self.cursor.fetchall()
 
     def thresholded_events_count(self):
         """
@@ -768,7 +936,10 @@ class EventStorageCursor:
         return count_orphaned + count_timeouted
 
 
-class incstats_decorator(object):
+class incstats_decorator:  # pylint: disable=locally-disabled,too-fewpublic-methods,invalid-name
+    """
+    Decorator for calculating usage statistics.
+    """
 
     def __init__(self, stat_name, increment = 1):
         self.stat_name = stat_name
@@ -1100,9 +1271,10 @@ class EventStorageService:
         exc_store = None
         for _ in range(2):
             try:
+                self.savepoint_create()
                 self.cursor.insert_event(idea_event)
                 self.savepoint_create()
-                return 
+                return
 
             except psycopg2.DataError as err:
                 self.savepoint_rollback()
@@ -1152,6 +1324,21 @@ class EventStorageService:
         self.cursor.delete_event(eventid)
         self.commit()
 
+    @incstats_decorator('query_direct')
+    @handle_db_exceptions
+    def query_direct(self, raw_query, idents = None, params = None):
+        """
+        This method is a convenience wrapper for underlying
+        :py:func:`mentat.services.eventstorage.EventStorageCursor.query_direct`
+        method.
+
+        It will automatically commit transaction for successfull database operation
+        and rollback the invalid one.
+        """
+        result = self.cursor.query_direct(raw_query, idents, params)
+        self.commit()
+        return result
+
     @incstats_decorator('count_events')
     @handle_db_exceptions
     def count_events(self, parameters = None, qname = None):
@@ -1182,6 +1369,21 @@ class EventStorageService:
         self.commit()
         return count, result
 
+    @incstats_decorator('search_events_aggr')
+    @handle_db_exceptions
+    def search_events_aggr(self, parameters = None, qtype = QTYPE_AGGREGATE, qname = None, dbtoplist = False):
+        """
+        This method is a convenience wrapper for underlying
+        :py:func:`mentat.services.eventstorage.EventStorageCursor.search_events_aggr`
+        method.
+
+        It will automatically commit transaction for successfull database operation
+        and rollback the invalid one.
+        """
+        result = self.cursor.search_events_aggr(parameters, qtype = qtype, qname = qname, dbtoplist = dbtoplist)
+        self.commit()
+        return result
+
     @incstats_decorator('search_column_with')
     @handle_db_exceptions
     def search_column_with(self, column, function = 'min'):
