@@ -7,6 +7,7 @@
 # Use of this source is governed by the MIT license, see LICENSE file.
 #-------------------------------------------------------------------------------
 
+
 import unittest
 from pprint import pprint
 
@@ -15,6 +16,7 @@ import datetime
 import mentat.stats.idea
 import mentat.datatype.sqldb
 
+
 class TestMentatStatsIdea(unittest.TestCase):
 
     #
@@ -351,7 +353,12 @@ class TestMentatStatsIdea(unittest.TestCase):
             )
         )
 
-    def test_05_evaluate_events(self):
+    def test_05_calc_timeline_cfg(self):
+        """
+        Test timeline config calculations.
+        """
+
+    def test_06_evaluate_events(self):
         """
         Perform the message evaluation tests.
         """
@@ -396,7 +403,7 @@ class TestMentatStatsIdea(unittest.TestCase):
             'severities': {'__unknown__': 6}
         })
 
-    def test_06_truncate_stats(self):
+    def test_07_truncate_stats(self):
         """
         Perform the basic operativity tests.
         """
@@ -453,7 +460,7 @@ class TestMentatStatsIdea(unittest.TestCase):
             }
         )
 
-    def test_07_group_events(self):
+    def test_08_group_events(self):
         """
         Perform the basic operativity tests.
         """
@@ -631,7 +638,7 @@ class TestMentatStatsIdea(unittest.TestCase):
             }
         )
 
-    def test_08_evaluate_event_groups(self):
+    def test_09_evaluate_event_groups(self):
         """
         Perform the basic operativity tests.
         """
@@ -647,7 +654,7 @@ class TestMentatStatsIdea(unittest.TestCase):
             pprint(result)
         self.assertTrue(result)
 
-    def test_09_merge_stats(self):
+    def test_10_merge_stats(self):
         """
         Perform the statistics aggregation tests.
         """
@@ -657,9 +664,9 @@ class TestMentatStatsIdea(unittest.TestCase):
         sts2 = mentat.stats.idea.evaluate_events(self.ideas_raw)
         sts3 = mentat.stats.idea.evaluate_events(self.ideas_raw)
 
-        result = mentat.stats.idea._merge_stats(sts1)
-        result = mentat.stats.idea._merge_stats(sts2, result)
-        result = mentat.stats.idea._merge_stats(sts3, result)
+        result = mentat.stats.idea._merge_stats(sts1)  # pylint: disable=locally-disabled,protected-access
+        result = mentat.stats.idea._merge_stats(sts2, result)  # pylint: disable=locally-disabled,protected-access
+        result = mentat.stats.idea._merge_stats(sts3, result)  # pylint: disable=locally-disabled,protected-access
 
         self.assertEqual(
             result,
@@ -694,7 +701,7 @@ class TestMentatStatsIdea(unittest.TestCase):
             }
         )
 
-    def test_10_aggregate_stat_groups(self):
+    def test_11_aggregate_stat_groups(self):
         """
         Perform the statistic group aggregation tests.
         """
@@ -749,5 +756,8 @@ class TestMentatStatsIdea(unittest.TestCase):
         self.assertEqual(result['dt_to'], datetime.datetime.fromtimestamp(timestamp+900))
 
 
+#-------------------------------------------------------------------------------
+
+
 if __name__ == "__main__":
     unittest.main()
