@@ -34,7 +34,7 @@ import vial.menu
 from vial.app import VialBlueprint
 from vial.view import ItemListView, ItemShowView, ItemCreateView, ItemUpdateView, ItemDeleteView, ItemEnableView, ItemDisableView, ItemObjectRelationView
 from vial.view.mixin import HTMLMixin, SQLAlchemyMixin
-from vial.blueprints.groups.forms import AdminCreateGroupForm, AdminUpdateGroupForm, UpdateGroupForm
+from vial.blueprints.groups.forms import AdminCreateGroupForm, AdminUpdateGroupForm, UpdateGroupForm, GroupSearchForm
 
 
 BLUEPRINT_NAME = 'groups'
@@ -106,6 +106,67 @@ class ListView(HTMLMixin, SQLAlchemyMixin, ItemListView):
         )
         return action_menu
 
+    @staticmethod
+    def get_search_form(request_args):
+        """
+        Must return instance of :py:mod:`flask_wtf.FlaskForm` appropriate for
+        searching given type of items.
+        """
+        return GroupSearchForm(
+            request_args,
+            meta = {'csrf': False}
+        )
+
+    @staticmethod
+    def build_query(query, model, form_args):
+        # Adjust query based on text search string.
+        if 'search' in form_args and form_args['search']:
+            query = query\
+                .filter(
+                    or_(
+                        model.name.like('%{}%'.format(form_args['search'])),
+                        model.description.like('%{}%'.format(form_args['search'])),
+                    )
+                )
+        # Adjust query based on lower time boudary selection.
+        if 'dt_from' in form_args and form_args['dt_from']:
+            query = query.filter(model.createtime >= form_args['dt_from'])
+        # Adjust query based on upper time boudary selection.
+        if 'dt_to' in form_args and form_args['dt_to']:
+            query = query.filter(model.createtime <= form_args['dt_to'])
+        # Adjust query based on user state selection.
+        if 'state' in form_args and form_args['state']:
+            if form_args['state'] == 'enabled':
+                query = query.filter(model.enabled == True)
+            elif form_args['state'] == 'disabled':
+                query = query.filter(model.enabled == False)
+            elif form_args['state'] == 'managed':
+                query = query.filter(model.managed == True)
+            elif form_args['state'] == 'notmanaged':
+                query = query.filter(model.managed == False)
+        # Adjust query based on record source selection.
+        if 'source' in form_args and form_args['source']:
+            query = query\
+                .filter(model.source == form_args['source'])
+        # Adjust query based on user membership selection.
+        if 'member' in form_args and form_args['member']:
+            query = query\
+                .join(model.members)\
+                .filter(model.members.any(id = form_args['member'].id))
+        # Adjust query based on user membership selection.
+        if 'manager' in form_args and form_args['manager']:
+            query = query\
+                .join(model.managers)\
+                .filter(model.managers.any(id = form_args['managers'].id))
+        if 'sortby' in form_args and form_args['sortby']:
+            sortmap = {
+                'createtime.desc': lambda x, y: x.order_by(y.createtime.desc()),
+                'createtime.asc': lambda x, y: x.order_by(y.createtime.asc()),
+                'name.desc': lambda x, y: x.order_by(y.name.desc()),
+                'name.asc': lambda x, y: x.order_by(y.name.asc())
+            }
+            query = sortmap[form_args['sortby']](query, model)
+        return query
 
 class ShowView(HTMLMixin, SQLAlchemyMixin, ItemShowView):
     """
