@@ -27,12 +27,14 @@ import flask_login
 import flask_principal
 import flask_mail
 from flask_babel import gettext, lazy_gettext, force_locale
+from sqlalchemy import or_
 
+import vial.const
 import vial.acl
 from vial.app import VialBlueprint
 from vial.view import ItemListView, ItemShowView, ItemCreateView, ItemUpdateView, ItemDeleteView, ItemEnableView, ItemDisableView, ItemObjectRelationView
 from vial.view.mixin import HTMLMixin, SQLAlchemyMixin
-from vial.blueprints.users.forms import CreateUserAccountForm, UpdateUserAccountForm, AdminUpdateUserAccountForm
+from vial.blueprints.users.forms import CreateUserAccountForm, UpdateUserAccountForm, AdminUpdateUserAccountForm, UserSearchForm
 
 
 BLUEPRINT_NAME = 'users'
@@ -103,6 +105,75 @@ class ListView(HTMLMixin, SQLAlchemyMixin, ItemListView):
         )
         return action_menu
 
+    @staticmethod
+    def get_search_form(request_args):
+        """
+        Must return instance of :py:mod:`flask_wtf.FlaskForm` appropriate for
+        searching given type of items.
+        """
+        roles = list(zip(flask.current_app.config['ROLES'], flask.current_app.config['ROLES']))
+        return UserSearchForm(
+            request_args,
+            meta = {'csrf': False},
+            choices_roles = roles
+        )
+
+    @staticmethod
+    def build_query(query, model, form_args):
+        # Adjust query based on text search string.
+        if 'search' in form_args and form_args['search']:
+            query = query\
+                .filter(
+                    or_(
+                        model.login.like('%{}%'.format(form_args['search'])),
+                        model.fullname.like('%{}%'.format(form_args['search'])),
+                        model.email.like('%{}%'.format(form_args['search'])),
+                    )
+                )
+        # Adjust query based on lower time boudary selection.
+        if 'dt_from' in form_args and form_args['dt_from']:
+            query = query.filter(model.createtime >= form_args['dt_from'])
+        # Adjust query based on upper time boudary selection.
+        if 'dt_to' in form_args and form_args['dt_to']:
+            query = query.filter(model.createtime <= form_args['dt_to'])
+        # Adjust query based on user state selection.
+        if 'state' in form_args and form_args['state']:
+            if form_args['state'] == 'enabled':
+                query = query.filter(model.enabled == True)
+            elif form_args['state'] == 'disabled':
+                query = query.filter(model.enabled == False)
+        # Adjust query based on user role selection.
+        if 'role' in form_args and form_args['role']:
+            if form_args['role'] == vial.const.NO_ROLE:
+                query = query.filter(model.roles == [])
+            else:
+                query = query.filter(model.roles.any(form_args['role']))
+        # Adjust query based on user membership selection.
+        if 'membership' in form_args and form_args['membership']:
+            query = query\
+                .join(model.memberships)\
+                .filter(model.memberships.any(id = form_args['membership'].id))
+        # Adjust query based on user membership selection.
+        if 'management' in form_args and form_args['management']:
+            query = query\
+                .join(model.managements)\
+                .filter(model.managements.any(id = form_args['management'].id))
+        if 'sortby' in form_args and form_args['sortby']:
+            sortmap = {
+                'createtime.desc': lambda x, y: x.order_by(y.createtime.desc()),
+                'createtime.asc': lambda x, y: x.order_by(y.createtime.asc()),
+                'login.desc': lambda x, y: x.order_by(y.login.desc()),
+                'login.asc': lambda x, y: x.order_by(y.login.asc()),
+                'fullname.desc': lambda x, y: x.order_by(y.fullname.desc()),
+                'fullname.asc': lambda x, y: x.order_by(y.fullname.asc()),
+                'email.desc': lambda x, y: x.order_by(y.email.desc()),
+                'email.asc': lambda x, y: x.order_by(y.email.asc()),
+                'logintime.desc': lambda x, y: x.order_by(y.logintime.desc()),
+                'logintime.asc': lambda x, y: x.order_by(y.logintime.asc()),
+            }
+            query = sortmap[form_args['sortby']](query, model)
+        return query
+
 
 class ShowView(HTMLMixin, SQLAlchemyMixin, ItemShowView):
     """
@@ -257,6 +328,8 @@ class MeView(ShowView):  # pylint: disable=locally-disabled,too-many-ancestors
 
     authentication = True
 
+    authorization = [vial.acl.PERMISSION_ANY]
+
     @classmethod
     def get_view_name(cls):
         return 'me'
