@@ -121,6 +121,8 @@ ROLES = [
 ]
 """List of valid user roles."""
 
+NO_ROLE = '__NO_ROLE__'
+"""Special constant for selecting users with no roles."""
 
 CFGKEY_MENU_MAIN_SKELETON = 'MENU_MAIN_SKELETON'
 """Configuration key name: Default application main menu skeleton."""
@@ -176,7 +178,7 @@ ICON_NAME_MISSING_ICON = 'missing-icon'
 """Name of the icon to display instead of missing icons."""
 
 
-FA_ICONS = {
+ICONS = {
 
     #
     # General icons.
@@ -243,6 +245,9 @@ FA_ICONS = {
     #
     # Action icons.
     #
+    'action-sort':         '<i class="fas fa-fw fa-sort"></i>',
+    'action-sort-asc':     '<i class="fas fa-fw fa-sort-up"></i>',
+    'action-sort-desc':    '<i class="fas fa-fw fa-sort-down"></i>',
     'action-more':         '<i class="fas fa-fw fa-cubes"></i>',
     'action-search':       '<i class="fas fa-fw fa-search"></i>',
     'action-show':         '<i class="fas fa-fw fa-eye"></i>',
