@@ -83,6 +83,10 @@ def _filter_menu_entries(entries, **kwargs):
         if entry.authorization:
             hideflag = False
             for authspec in entry.authorization:
+                # When accessing static files, there is no identity in g so the can() function would raise an exception.
+                if not 'identity' in flask.g:
+                    hideflag = True
+                    break
                 # Authorization rules may be specified as instances of flask_principal.Permission.
                 if isinstance(authspec, flask_principal.Permission):
                     if not authspec.can():
