@@ -118,6 +118,21 @@ class BaseView(flask.views.View):
         """
         raise NotImplementedError()
 
+    @classmethod
+    def get_view_endpoint_name(cls):
+        """
+        Return unique name for the view endpoint. Name must be unique in the namespace of
+        parent blueprint/module and should contain only characters ``[a-z0-9]``.
+        It will be used for generating endpoint name for the view.
+
+        *This method does not have any default implementation and must be overridden
+        by a subclass.*
+
+        :return: Name for the view.
+        :rtype: str
+        """
+        return cls.get_view_name()
+
     @classmethod
     def get_view_endpoint(cls):
         """
@@ -129,7 +144,10 @@ class BaseView(flask.views.View):
         :return: Routing endpoint for the view within the whole application.
         :rtype: str
         """
-        return '{}.{}'.format(cls.module_name, cls.get_view_name())
+        return '{}.{}'.format(
+            cls.module_name,
+            cls.get_view_endpoint_name()
+        )
 
     @classmethod
     def get_view_url(cls, **kwargs):
@@ -456,7 +474,7 @@ class RenderableView(BaseView):  # pylint: disable=locally-disabled,abstract-met
         if cls.module_name:
             return '{}/{}.html'.format(
                 cls.module_name,
-                cls.get_view_name()
+                cls.get_view_endpoint_name()
             )
         raise RuntimeError("Unable to guess default view template, because module name was not yet set.")
 
@@ -620,6 +638,13 @@ class BaseLoginView(SimpleView):
                 )
                 self.abort(500)
 
+            if not user:
+                self.flash(
+                    gettext('You have entered wrong login credentials.'),
+                    vial.const.FLASH_FAILURE
+                )
+                self.abort(403)
+
             if not user.enabled:
                 self.flash(
                     flask.Markup(gettext(
@@ -804,10 +829,10 @@ class BaseSearchView(RenderableView, VialUtils):
         )
         breadcrumbs_menu.add_entry(
             'endpoint',
-            cls.get_view_name(),
+            cls.get_view_endpoint_name(),
             endpoint = '{}.{}'.format(
                 cls.module_name,
-                cls.get_view_name()
+                cls.get_view_endpoint_name()
             )
         )
         return breadcrumbs_menu
@@ -1084,7 +1109,6 @@ class ItemListView(RenderableView):  # pylint: disable=locally-disabled,abstract
     Base class for item *list* views. These views provide quick and simple access
     to lists of all objects.
     """
-
     @classmethod
     def get_view_name(cls):
         """*Implementation* of :py:func:`vial.view.BaseView.get_view_name`."""
@@ -1104,7 +1128,10 @@ class ItemListView(RenderableView):  # pylint: disable=locally-disabled,abstract
         action_menu.add_entry(
             'endpoint',
             'list',
-            endpoint = '{}.list'.format(cls.module_name)
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name()
+            )
         )
         return action_menu
 
@@ -1122,6 +1149,38 @@ class ItemListView(RenderableView):  # pylint: disable=locally-disabled,abstract
         """
         return None
 
+    @staticmethod
+    def get_query_parameters(form, request_args):
+        """
+        Get query parameters by comparing contents of processed form data and
+        original request arguments. Result of this method can be used for generating
+        modified URLs back to current request. One of the use cases is the result
+        pager/paginator.
+        """
+        params = {}
+        for arg in request_args:
+            if form and getattr(form, arg, None) and arg in request_args:
+                # Handle multivalue request arguments separately
+                # Resources:
+                #   http://flask.pocoo.org/docs/1.0/api/#flask.Request.args
+                #   http://werkzeug.pocoo.org/docs/0.14/datastructures/#werkzeug.datastructures.MultiDict
+                try:
+                    if form.is_multivalue(arg):
+                        params[arg] = request_args.getlist(arg)
+                    else:
+                        params[arg] = request_args[arg]
+                except AttributeError:
+                    params[arg] = request_args[arg]
+        return params
+
+    @staticmethod
+    def get_search_form(request_args):
+        """
+        Must return instance of :py:mod:`flask_wtf.FlaskForm` appropriate for
+        searching given type of items.
+        """
+        return None
+
     def search(self, form_args):
         """
         Perform actual search with given form arguments.
@@ -1136,10 +1195,43 @@ class ItemListView(RenderableView):  # pylint: disable=locally-disabled,abstract
         List of all items will be retrieved from database and injected into template
         to be displayed to the user.
         """
-        items = self.search({})
+        form = self.get_search_form(flask.request.args)
+        flask.g.search_form = form
+
+        if form:
+            form_data = form.data
+            items = []
+            if form.validate():
+                form_data = form.data
+                items = self.search(form_data)
+
+                self.response_context.update(
+                    form_data = form_data,
+                    searched = True,
+                    items = items,
+                    items_count = len(items)
+                )
+
+            # Not all search forms support result paging.
+            if 'page' in form_data:
+                self.response_context.update(
+                    pager_index_low = ((form_data['page'] - 1) * form_data['limit']) + 1,
+                    pager_index_high = ((form_data['page'] - 1) * form_data['limit']) + len(items),
+                    pager_index_limit = ((form_data['page'] - 1) * form_data['limit']) + form_data['limit']
+                )
+        else:
+            items = self.search({})
+            self.response_context.update(
+                items = items
+            )
 
         self.response_context.update(
-            items = items
+            query_params = self.get_query_parameters(
+                form,
+                flask.request.args
+            ),
+            search_widget_item_limit = 3,
+            form = form
         )
 
         self.do_before_response()
@@ -1171,6 +1263,12 @@ class ItemShowView(RenderableView):  # pylint: disable=locally-disabled,abstract
     @classmethod
     def get_view_url(cls, **kwargs):
         """*Implementation* of :py:func:`vial.view.BaseView.get_view_url`."""
+        if 'item' not in kwargs or not kwargs['item']:
+            raise ValueError(
+                "Missing item parameter for show URL view '{}'". format(
+                    cls.get_view_endpoint()
+                )
+            )
         return flask.url_for(
             cls.get_view_endpoint(),
             item_id = kwargs['item'].get_id()
@@ -1209,13 +1307,19 @@ class ItemShowView(RenderableView):  # pylint: disable=locally-disabled,abstract
         action_menu.add_entry(
             'endpoint',
             'list',
-            endpoint = '{}.list'.format(cls.module_name),
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name().replace('show', 'list')
+            ),
             paramlist = []
         )
         action_menu.add_entry(
             'endpoint',
             'show',
-            endpoint = '{}.show'.format(cls.module_name)
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name()
+            )
         )
         return action_menu
 
@@ -1486,12 +1590,18 @@ class ItemCreateView(ItemActionView):  # pylint: disable=locally-disabled,abstra
         breadcrumbs_menu.add_entry(
             'endpoint',
             'list',
-            endpoint = '{}.list'.format(cls.module_name)
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name().replace('create', 'list')
+            ),
         )
         breadcrumbs_menu.add_entry(
             'endpoint',
             'create',
-            endpoint = '{}.create'.format(cls.module_name)
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name()
+            )
         )
         return breadcrumbs_menu
 
@@ -1977,17 +2087,26 @@ class ItemUpdateView(ItemActionView):  # pylint: disable=locally-disabled,abstra
         breadcrumbs_menu.add_entry(
             'endpoint',
             'list',
-            endpoint = '{}.list'.format(cls.module_name)
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name().replace('update', 'list')
+            )
         )
         breadcrumbs_menu.add_entry(
             'endpoint',
             'show',
-            endpoint = '{}.show'.format(cls.module_name)
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name().replace('update', 'show')
+            )
         )
         breadcrumbs_menu.add_entry(
             'endpoint',
             'update',
-            endpoint = '{}.update'.format(cls.module_name)
+            endpoint = '{}.{}'.format(
+                cls.module_name,
+                cls.get_view_endpoint_name()
+            )
         )
         return breadcrumbs_menu
 
@@ -2363,7 +2482,7 @@ class ItemObjectRelationView(ItemChangeView):  # pylint: disable=locally-disable
         :rtype: str
         """
         if cls.module_name:
-            return '{}/{}.html'.format(cls.module_name, cls.get_view_name())
+            return '{}/{}.html'.format(cls.module_name, cls.get_view_endpoint_name())
         raise RuntimeError("Unable to guess default view template, because module name was not yet set.")
 
     @classmethod
